package cz.muni.fi.pa165.facade;

import cz.muni.fi.pa165.dto.MatchDTO;
import cz.muni.fi.pa165.entity.MatchEntity;
import cz.muni.fi.pa165.entity.PlayerEntity;
import cz.muni.fi.pa165.entity.TeamEntity;
import cz.muni.fi.pa165.services.BeanMappingService;
import cz.muni.fi.pa165.services.match.MatchService;
import cz.muni.fi.pa165.services.team.TeamService;
import cz.muni.fi.pa165.services.tournament.TournamentService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.transaction.Transactional;
import java.util.HashSet;
import java.util.List;
import java.util.UUID;

/**
 * @author Dominik Kozubík
 */
@Service
@Transactional
public class MatchFacadeImpl implements MatchFacade {

    @Autowired
    private MatchService matchService;

    @Autowired
    private BeanMappingService beanMappingService;

    @Autowired
    private TeamService teamService;

    @Autowired
    private TournamentService tournamentService;

    @Override
    public List<MatchDTO> findAllMatches() {
        return beanMappingService.mapTo(matchService.findAll(), MatchDTO.class);
    }

    @Override
    public MatchDTO getMatchById(String uuid) {
        return beanMappingService.mapTo(matchService.findById(uuid), MatchDTO.class);
    }

    @Override
    public String createMatch(MatchDTO matchDTO) {
        MatchEntity match = new MatchEntity();
        match.setId(UUID.randomUUID().toString());

        var mappedPlayers = new HashSet<PlayerEntity>();
        for (var player : matchDTO.getPlayerList()) {
            mappedPlayers.add(beanMappingService.mapTo(player, PlayerEntity.class));
        }
        match.setPlayerList(mappedPlayers);

        mappedPlayers.clear();
        for (var player : matchDTO.getPlayers()) {
            mappedPlayers.add(beanMappingService.mapTo(player, PlayerEntity.class));
        }
        match.setPlayers(mappedPlayers);

        var mappedTeams = new HashSet<TeamEntity>();
        for (var teamID : matchDTO.getTeamsIds()) {
            mappedTeams.add(teamService.findById(teamID));
        }
        match.setTeams(mappedTeams);
        match.setTournament(tournamentService.findById(matchDTO.getTournamentId()));

        matchService.create(match);

        return match.getId();
    }

    @Override
    public void delete(String uuid) {
        var match = new MatchEntity();
        match.setId(uuid);
        matchService.remove(match);
    }
}
